
# Steps for Analysis

## 0. Setup Python Environment
This script only works with Python 2, so ensure that Python 2 is set up first.

---

## 1. `setup_analysis_components.sh`

Make sure to set the user inputs at the top of the script.

If `frameperjob` is less than the total number of frames, this will create multiple workers to identify connected components.

---

## 2. Run Individual Workers or `submit_analysis_components.sh`

This step identifies connected components.

If there is only a single worker, it is recommended to go to `dir.components.work/worker.1` and execute the following:

```bash
./run_analysis_components.sh
```

---

## 3. `collect_descriptors.sh`

Run this after all individual worker jobs are completed.

Make sure to set the user inputs at the top of the script.

This will process all connected components identified in the previous step and compute descriptors for these components. It does not work with graphs, as they are quite expensive and can cause the isomorphism check to hang.

The script uses descriptors similar to those in **Molanal**, but with improvements.

- This code also enumerates unique angle types and considers those as well.  
  For example:  
  `D = (Na Nb Na-a Nb-b Na-b Na-a-a Na-a-b N-a-b-a ...)`

---

## 4. `final_cleanup.sh`

This script performs final analysis and identifies reaction states or combinations of structures. Each frame is assigned a reaction state of a particular type, which is indexed automatically.

Two files of interest in the `outputs` directory:

- `rxnstates.txt`: Provides the reaction state of each frame, along with the component IDs that comprise it. These IDs follow the numbering scheme in `lib.structures/configs`.

- `unique.rxnstates.txt`: Similar to `rxnstates.txt`, but only the unique states are listed.

---

## 5. `sort_library.sh`

This script applies an optional sort on the identified structures, ordering them by increasing mass, which is useful for finding small products.

It generates a mapping in `sort_output.txt` between the auto-generated structure type indexing (in the second column) and a mass-ordered list.

Additionally, it creates a complementary directory in `outputs/lib.structures/sorted_configs` for easy visualization of the mass-ordered set of products.

